/* ============================================================================
 *  "Hello OpenGL ES" OpenGL ES code.
 *
 *  Eurographics 2006 course on mobile graphics.
 *
 *  Copyright: Jani Vaarala
 * ============================================================================
*/

#include <e32base.h>
#include "EGTriangleGL.h"

static const GLbyte vertices[3 * 3] =
{
     -1,    1,    0,
      1,   -1,    0,
      1,    1,    0
};

static const GLubyte colors[3 * 4] =
{
        255,      0,    0,    255,
          0,    255,    0,    255,
          0,      0,  255,    255
};

static void initGLES()
{
    glClearColor        (0.f,0.f,0.1f,1.f);
    glDisable           (GL_DEPTH_TEST);
    glMatrixMode        (GL_PROJECTION);
    glFrustumf          (-1.f,1.f,-1.f,1.f,3.f,1000.f);
    glMatrixMode        (GL_MODELVIEW);
    glShadeModel        (GL_SMOOTH);
    glVertexPointer     (3,GL_BYTE,0,vertices);
    glColorPointer      (4,GL_UNSIGNED_BYTE,0,colors);
    glEnableClientState (GL_VERTEX_ARRAY);
    glEnableClientState (GL_COLOR_ARRAY);
}

/************************************************************
 * Initialize OpenGL ES context and initial OpenGL ES state *
 ************************************************************/

void CEGTriangleGL::Construct(RWindow aWin)
{
    iWin = aWin;

    iEglDisplay = eglGetDisplay(EGL_DEFAULT_DISPLAY);
    if(iEglDisplay == NULL )
                User::Exit(-1);

    if(eglInitialize(iEglDisplay,NULL,NULL) == EGL_FALSE)
                User::Exit(-1);

    EGLConfig  config,colorDepth;
    EGLint     numOfConfigs = 0; 
        
    switch( iWin.DisplayMode() )
    {
        case (EColor4K):   { colorDepth = 12; break; }
        case (EColor64K):  { colorDepth = 16; break; }
        case (EColor16M):  { colorDepth = 24; break; }
        default:
                             colorDepth = 32;
    }

    EGLint attrib_list[] = {       EGL_BUFFER_SIZE, colorDepth,
                                   EGL_DEPTH_SIZE,  15,
                                   EGL_NONE                                 };

    if(eglChooseConfig(iEglDisplay,attrib_list,&config,1,&numOfConfigs ) == EGL_FALSE)
                User::Exit(-1);

    iEglSurface = eglCreateWindowSurface( iEglDisplay, config, &iWin, NULL );
    if( iEglSurface == NULL )
                User::Exit(-1);

    iEglContext = eglCreateContext( iEglDisplay, config, EGL_NO_CONTEXT, NULL );
    
    if( iEglContext == NULL )
                User::Exit(-1);

    if( eglMakeCurrent( iEglDisplay, iEglSurface, iEglSurface, iEglContext ) == EGL_FALSE )
                User::Exit(-1);

    /* Create a periodic timer for display refresh */
    iPeriodic = CPeriodic::NewL( CActive::EPriorityIdle );
    iPeriodic->Start( 100, 100, TCallBack( CEGTriangleGL::DrawCallback, this ) );

    initGLES();
    iFrame = 0;
}

CEGTriangleGL::~CEGTriangleGL()
{
}

/************************************************************
 * Draw callback executed regularly by a timer callback
 ************************************************************/

TInt CEGTriangleGL::DrawCallback( TAny* aInstance )
{
    CEGTriangleGL* instance = (CEGTriangleGL*) aInstance;
        
    glClear                         (GL_COLOR_BUFFER_BIT);
    glLoadIdentity          ();
    glTranslatef            (0,0,-5.f);
    glDrawArrays            (GL_TRIANGLES,0,3);

    eglSwapBuffers          (instance->iEglDisplay, instance->iEglSurface);

    /* To keep the background light on */
    if (!(instance->iFrame%100))        User::ResetInactivityTime();
        instance->iFrame++;

    return 0;
}
